
package edu.uthscsa.ric.plugins.mangoplugin;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.net.URI;
import java.net.URL;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import edu.uthscsa.ric.utilities.AppLogger;
import edu.uthscsa.ric.volume.Coordinate;
import edu.uthscsa.ric.volume.EditableHeader;
import edu.uthscsa.ric.volume.ImageDescription;
import edu.uthscsa.ric.volume.ImageDimensions;
import edu.uthscsa.ric.volume.ImageRange;
import edu.uthscsa.ric.volume.ImageTransform;
import edu.uthscsa.ric.volume.ImageType;
import edu.uthscsa.ric.volume.ReadableHeader;
import edu.uthscsa.ric.volume.VoxelDimensions;
import edu.uthscsa.ric.volume.WritableHeader;


/**
 * This plugin implements the EditableHeader, ReadableHeader and WritableHeader interfaces. A plugin that implements one or more of these interfaces can extend
 * Mango file support. This fictitious binary format stores image dimensions and an orientation code.
 * 
 * @author michaelmartinez
 *
 */
public class ExampleReaderWriter implements EditableHeader, ReadableHeader, WritableHeader {

	private URI headerURI;
	private int orientationCode;
	private int seriesLength;
	private int xDim;
	private int yDim;
	private int zDim;

	public static final String EXTENSION = ".example";
	public static final String FIELD_ORIENTATION = "Orientation";
	public static final String FIELD_SERIES = "Series Length";
	public static final String FIELD_X = "X Dimension";
	public static final String FIELD_Y = "Y Dimension";
	public static final String FIELD_Z = "Z Dimension";
	public static final String ORIENTATION_STRING_0 = "XYZ+++";
	public static final String ORIENTATION_STRING_1 = "XYZ---";
	public static final byte[] MAGIC_NUM = { 101, 120, 97, 109 }; // "exam"
	public static final int ORIENTATION_CODE_0 = 0;
	public static final int ORIENTATION_CODE_1 = 1;



	/***
	 * ReadableHeader interface
	 * 
	 * @deprecated
	 ***/

	@Deprecated
	@Override
	public ByteBuffer getImage() {
		return null;
	}



	@Override
	public ImageDescription getImageDescription() {
		return new ImageDescription();
	}



	@Override
	public ImageDimensions getImageDimensions() {
		final ImageDimensions id = new ImageDimensions(xDim, yDim, zDim, seriesLength);
		id.setImageOffset(MAGIC_NUM.length + (5 * 4));
		return id;
	}



	@Override
	public URI getImageFile() {
		return headerURI; // header is prepended to image data, so it's the same file
	}



	@Override
	public ImageRange getImageRange() {
		return new ImageRange(0, 0, 0, 0);
	}



	@Override
	public ImageType getImageType() {
		return new ImageType(4, ImageType.BYTE_TYPE_FLOAT, 4 * 8, true);
	}



	@Override
	public String getOrientation() {
		if (orientationCode == 0) {
			return ORIENTATION_STRING_0;
		} else {
			return ORIENTATION_STRING_1;
		}
	}



	@Override
	public int getOrientationCertainty() {
		return ReadableHeader.ORIENTATION_CERTAINTY_HIGH;
	}



	@Override
	public Coordinate getOrigin() {
		return new Coordinate(0, 0, 0);
	}



	@Override
	public String getPreferredFileExtension() {
		return EXTENSION;
	}



	@Override
	public float[] getPreviewImage() {
		return null;
	}



	@Override
	public ImageTransform[] getTransforms() {
		return null;
	}



	@Override
	public VoxelDimensions getVoxelDimensions() {
		return new VoxelDimensions(1, 1, 1, 1);
	}



	@Override
	public boolean isThisFormat(final File file) {
		boolean success = false;

		BufferedInputStream fileIn = null;
		final byte[] buffer = new byte[MAGIC_NUM.length];

		try {
			fileIn = new BufferedInputStream(new FileInputStream(file));

			int bytesRead = fileIn.read(buffer);
			int totalBytesRead = bytesRead;

			while ((totalBytesRead > 0) && (totalBytesRead < buffer.length)) {
				bytesRead = fileIn.read(buffer, totalBytesRead, buffer.length - totalBytesRead);
				totalBytesRead += bytesRead;
			}

			success = Arrays.equals(buffer, MAGIC_NUM);
		} catch (final IOException ex) {
			AppLogger.error(ex);
		} finally {
			try {
				if (fileIn != null) {
					fileIn.close();
				}
			} catch (final IOException ex) {
				AppLogger.warn(ex);
			}
		}

		return success;
	}



	@Override
	public void readHeader(final File file, final File tempDir) {
		readHeader(file.toURI(), tempDir);
	}



	@Override
	public void readHeader(final URI uri, final File tempDir) {
		headerURI = uri;

		BufferedInputStream input = null;
		byte[] buffer = null;

		try {
			buffer = new byte[MAGIC_NUM.length + (5 * 4)]; // header consists of 5 ints + MAGIC_NUM
			input = new BufferedInputStream(new FileInputStream(new File(uri)));

			// skip magic num
			final int bytesToSkip = MAGIC_NUM.length;
			long bytesSkipped = input.skip(bytesToSkip);
			long totalBytesSkipped = bytesSkipped;

			while (totalBytesSkipped < bytesToSkip) {
				bytesSkipped = input.skip(bytesToSkip - totalBytesSkipped);
				totalBytesSkipped += bytesSkipped;
			}

			// read field data
			int bytesRead = input.read(buffer);
			int totalBytesRead = bytesRead;

			while ((totalBytesRead > 0) && (totalBytesRead < buffer.length)) {
				bytesRead = input.read(buffer, totalBytesRead, buffer.length - totalBytesRead);
				totalBytesRead += bytesRead;
			}
		} catch (final IOException ex) {
			AppLogger.error(ex);
		} finally {
			try {
				if (input != null) {
					input.close();
				}
			} catch (final IOException ex) {
				AppLogger.error(ex);
			}
		}

		final ByteBuffer byteBuffer = ByteBuffer.wrap(buffer);
		byteBuffer.order(ByteOrder.LITTLE_ENDIAN);

		xDim = byteBuffer.getInt();
		yDim = byteBuffer.getInt();
		zDim = byteBuffer.getInt();
		seriesLength = byteBuffer.getInt();
		orientationCode = byteBuffer.getInt();
	}



	@Override
	public String toString() {
		final StringBuffer sb = new StringBuffer();
		sb.append(FIELD_X + ": " + xDim + "\n");
		sb.append(FIELD_Y + ": " + yDim + "\n");
		sb.append(FIELD_Z + ": " + zDim + "\n");
		sb.append(FIELD_SERIES + ": " + seriesLength + "\n");
		sb.append(FIELD_ORIENTATION + ": " + orientationCode);
		return sb.toString();
	}



	@Override
	public ByteBuffer[] getImageBuffers() {
		return null;
	}



	@Override
	public String getSliceMetadata(final int slice, final int seriesPoint) {
		return toString();
	}



	@Override
	public short[] getSliceAnnotation(final int slice, final int seriesPoint) {
		return null;
	}



	/*** WritableHeader interface ***/

	@Override
	public boolean canCompress() {
		return false;
	}



	@Override
	public File formatImageFile(final File headerFile) {
		return headerFile;
	}



	@Override
	public int[] getSupportedBytesForType(final int type) {
		if (type == ImageType.BYTE_TYPE_INTEGER) {
			return new int[] {};
		} else if (type == ImageType.BYTE_TYPE_INTEGER_UNSIGNED) {
			return new int[] {};
		} else if (type == ImageType.BYTE_TYPE_FLOAT) {
			return new int[] { 4 };
		} else {
			return null;
		}
	}



	@Override
	public String[] getSupportedOrientations() {
		return new String[] { ORIENTATION_STRING_0, ORIENTATION_STRING_1 };
	}



	@Override
	public int[] getSupportedTypesForBytes(final int bytes) {
		if (bytes == 1) {
			return new int[] {};
		} else if (bytes == 2) {
			return new int[] {};
		} else if (bytes == 4) {
			return new int[] { ImageType.BYTE_TYPE_FLOAT };
		} else {
			return null;
		}
	}



	@Override
	public boolean isSupportedBigEndian() {
		return false;
	}



	@Override
	public boolean isSupportedLittleEndian() {
		return true;
	}



	@Override
	public boolean isSupportedMultipleDataScales() {
		return false;
	}



	@Override
	public boolean willWriteImage() {
		return false;
	}



	@Override
	public byte[] writeHeader(final ImageDimensions id, final VoxelDimensions vd, final ImageType it, final ImageRange ir, final ImageDescription in,
			final boolean compress, final Coordinate origin, final String orientation, final ImageTransform[] transforms, final File headerFile,
			final File imageFile, final ByteBuffer[] imageBuffers, final File tempDir) {
		if (id != null) {
			xDim = id.getCols();
			yDim = id.getRows();
			zDim = id.getSlices();
			seriesLength = id.getTimepoints();
		}

		if (orientation != null) {
			if (orientation.equals(ORIENTATION_STRING_0)) {
				orientationCode = 0;
			} else if (orientation.equals(ORIENTATION_STRING_1)) {
				orientationCode = 1;
			}
		}

		final ByteBuffer byteBuffer = ByteBuffer.wrap(new byte[MAGIC_NUM.length + (5 * 4)]);
		byteBuffer.order(ByteOrder.LITTLE_ENDIAN);
		byteBuffer.put(MAGIC_NUM);
		byteBuffer.putInt(xDim);
		byteBuffer.putInt(yDim);
		byteBuffer.putInt(zDim);
		byteBuffer.putInt(seriesLength);
		byteBuffer.putInt(orientationCode);

		return byteBuffer.array();
	}



	@Override
	public byte[][] writeSeriesHeaders(final ImageDimensions id, final VoxelDimensions vd, final ImageType it, final ImageRange ir, final ImageDescription in,
			final boolean compress, final Coordinate origin, final String orientation, final ImageTransform[] transforms, final File headerFile,
			final File imageFile, final ByteBuffer[] imageBuffers, final File tempDir) throws Exception {
		return null;
	}



	@Override
	public boolean isSupportedDataScaleSlope() {
		return false;
	}



	@Override
	public boolean isSupportedDataScaleIntercept() {
		return false;
	}



	/*** EditableHeader interface ***/

	@Override
	public Map<String, List<String>> getEditableFieldValues() {
		final Map<String, List<String>> map = new LinkedHashMap<String, List<String>>();

		map.put(FIELD_X, Arrays.asList(new String[] { String.valueOf(xDim) }));
		map.put(FIELD_Y, Arrays.asList(new String[] { String.valueOf(yDim) }));
		map.put(FIELD_Z, Arrays.asList(new String[] { String.valueOf(zDim) }));
		map.put(FIELD_SERIES, Arrays.asList(new String[] { String.valueOf(seriesLength) }));
		map.put(FIELD_ORIENTATION, Arrays.asList(new String[] { String.valueOf(orientationCode) }));

		return map;
	}



	@Override
	public Map<String, List<CodeOption>> getHumanReadableFieldOptions() {
		final Map<String, List<EditableHeader.CodeOption>> map = new LinkedHashMap<String, List<EditableHeader.CodeOption>>();

		final List<EditableHeader.CodeOption> dataTypeOptions = new ArrayList<EditableHeader.CodeOption>();
		dataTypeOptions.add(new EditableHeader.CodeOption(String.valueOf(ORIENTATION_CODE_0), ORIENTATION_STRING_0));
		dataTypeOptions.add(new EditableHeader.CodeOption(String.valueOf(ORIENTATION_CODE_1), ORIENTATION_STRING_1));
		Collections.sort(dataTypeOptions);
		map.put(FIELD_ORIENTATION, dataTypeOptions);

		return map;
	}



	@Override
	public boolean isTextEditableHeader() {
		return false;
	}



	@Override
	public void setEditableFieldValue(final String name, final String value, final int index) {
		if (name.equals(FIELD_X)) {
			xDim = Integer.parseInt(value);
		} else if (name.equals(FIELD_Y)) {
			yDim = Integer.parseInt(value);
		} else if (name.equals(FIELD_Z)) {
			zDim = Integer.parseInt(value);
		} else if (name.equals(FIELD_SERIES)) {
			seriesLength = Integer.parseInt(value);
		} else if (name.equals(FIELD_ORIENTATION)) {
			orientationCode = Integer.parseInt(value);
		}
	}



	@Override
	public void writeHeader() {
		final byte[] header = writeHeader(null, null, null, null, null, false, null, null, null, null, null, null, null);
		RandomAccessFile raf = null;

		try {
			raf = new RandomAccessFile(new File(headerURI), "rws");
			raf.seek(0);
			raf.write(header);
		} catch (final IOException ex) {
			AppLogger.error(ex);
		} finally {
			try {
				if (raf != null) {
					raf.close();
				}
			} catch (final IOException ex) {
				AppLogger.error(ex);
			}
		}
	}



	/*** MangoPluginType interface ***/

	@Override
	public String getMinimumVersionSupported() {
		return null;
	}



	@Override
	public String getPluginName() {
		return "Example Format";
	}



	@Override
	public URL getPluginURL() {
		return null;
	}



	@Override
	public String getVersion() {
		return null;
	}



	@Override
	public boolean hasNewerVersion() {
		return false;
	}



	@Override
	public List<String> getAnonymizableFields() {
		return null;
	}



	@Override
	public Map<String, List<String>> anonymize() {
		return null;
	}
}
